<?php

declare(strict_types=1);

namespace Drupal\ckeditor5_find_replace_filter\Plugin\Filter;

use Drupal\Core\Form\FormStateInterface;
use Drupal\filter\FilterProcessResult;
use Drupal\filter\Plugin\FilterBase;

/**
 *
 * @Filter(
 *   id = "ckeditor5_find_replace_filter",
 *   title = @Translation("Ckeditor5 Find and Replace Filter"),
 *   type = Drupal\filter\Plugin\FilterInterface::TYPE_TRANSFORM_IRREVERSIBLE,
 *   settings = {},
 * )
 */
final class Ckeditor5FindReplaceFilter extends FilterBase {

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state): array {
    $form['find_text'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Find Text'),
      '#default_value' => $this->settings['find_text'] ?? '',
      '#description' => $this->t('Find Text. Use (,) operator to find multiple texts.'),
      '#required' => TRUE,
    ];

    $form['replace_text'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Replace Text'),
      '#default_value' => $this->settings['replace_text'] ?? '',
      '#description' => $this->t('Replacement Text. Use (,) operator to replace multiple texts.'),
      '#required' => TRUE,
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function process($text, $langcode): FilterProcessResult {
    $find_texts = array_map('trim', explode(',', $this->settings['find_text'] ?? ''));
    $replace_texts = array_map('trim', explode(',', $this->settings['replace_text'] ?? ''));

    // Ensure that the number of find and replace texts match.
    if (count($find_texts) !== count($replace_texts)) {
      return new FilterProcessResult($text);
    }

    // Sanitize find_texts and replace_texts.
    $find_texts = array_map('htmlspecialchars', $find_texts);
    $replace_texts = array_map('htmlspecialchars', $replace_texts);

    // Perform the replacement.
    $updatedHTML = str_replace($find_texts, $replace_texts, $text);

    return new FilterProcessResult($updatedHTML);
  }

  /**
   * {@inheritdoc}
   */
  public function setConfiguration(array $configuration) {
    if (isset($configuration['settings']['find_text'])) {
      $configuration['settings']['find_text'] = htmlspecialchars($configuration['settings']['find_text']);
    }
    if (isset($configuration['settings']['replace_text'])) {
      $configuration['settings']['replace_text'] = htmlspecialchars($configuration['settings']['replace_text']);
    }
    return parent::setConfiguration($configuration);
  }
}
